@@ -21,7 +21,7 @@ use File::Path qw/mkpath/;
 use File::Temp qw/tempfile/;
 use IO::Handle;
 
-our $VERSION = '1.16';
+our $VERSION = '1.17';
 
 # Default options
 my %default_params = ( MKPATH => 0, template => ".tmp.XXXXXXXXXX" );
@@ -271,7 +271,7 @@ sub _resolve {
   if ( exists $params_ref->{mode} ) {
     croak 'invalid mode data'
       if !defined $params_ref->{mode}
-        or $params_ref->{mode} !~ m/^\d+$/;
+        or $params_ref->{mode} !~ m/^[0-9]+$/;
 
     my $count = chmod( $params_ref->{mode}, $tmp_filename );
     if ( $count != 1 ) {
@@ -289,6 +289,20 @@ sub _resolve {
     }
   }
 
+  if ( exists $params_ref->{mtime} ) {
+    croak 'invalid mtime data'
+      if ! defined $params_ref->{mtime}
+        or $params_ref->{mtime} !~ m/^[0-9]+$/;
+
+    my ( $file_atime ) = ( stat $tmp_filename )[ 8 ];
+    my $count = utime( $file_atime, $params_ref->{mtime}, $tmp_filename );
+    if ( $count != 1 ) {
+      my $save_errstr = $!;
+      _cleanup( $tmp_fh, $tmp_filename );
+      die "unable to utime temporary file: $save_errstr\n";
+    }
+  }
+
   unless ( rename( $tmp_filename, $params_ref->{file} ) ) {
     my $save_errstr = $!;
     _cleanup( $tmp_fh, $tmp_filename );
@@ -370,7 +384,7 @@ sub _set_ownership {
   my ( $user_name, $group_name ) = split /[:.]/, $owner, 2;
 
   my ( $login, $pass, $user_uid, $user_gid );
-  if ( $user_name =~ m/^(\d+)$/ ) {
+  if ( $user_name =~ m/^([0-9]+)$/ ) {
     $uid = $1;
   } else {
     ( $login, $pass, $user_uid, $user_gid ) = getpwnam($user_name)
@@ -380,7 +394,7 @@ sub _set_ownership {
 
   # Only customize group if have something from caller
   if ( defined $group_name ) {
-    if ( $group_name =~ m/^(\d+)$/ ) {
+    if ( $group_name =~ m/^([0-9]+)$/ ) {
       $gid = $group_name;
     } else {
       my ( $group_name, $pass, $group_gid ) = getgrnam($group_name)
@@ -640,6 +654,11 @@ to the file. Usual throwing of error.
   ...->write_file({ ..., owner => '0:0' });
   ...->write_file({ ..., owner => 'user:somegroup' });
 
+=item B<mtime> => I<mtime>
+
+Accepts C<mtime> timestamp for C<utime> to be applied to the file.
+Usual throwing of error.
+
 =item B<tmpdir> => I<directory>
 
 If set to a directory, the temporary file will be written to this
@@ -682,7 +701,7 @@ Supply a C<LAYER> argument to C<binmode>. Enables B<BINMODE>.
 
   # just binmode (binary data)
   ...->write_file({ ..., BINMODE => 1 });
-  
+
   # custom binmode layer
   ...->write_file({ ..., binmode_layer => ':utf8' });
 
@@ -724,10 +743,10 @@ inode of the file being operated on:
 
   % touch afile
   % ln afile afilehardlink
-  % ls -i afile*          
+  % ls -i afile*
   3725607 afile         3725607 afilehardlink
   % perl -MFile::AtomicWrite -e \
-    'File::AtomicWrite->write_file({file =>"afile",input=>\"foo"})' 
+    'File::AtomicWrite->write_file({file =>"afile",input=>\"foo"})'
   % ls -i afile*
   3725622 afile         3725607 afilehardlink
 
@@ -738,7 +757,10 @@ locations.
 Some filesystems may also require a fsync call on a filehandle of the
 directory containing the file (see fsync(2) on RHEL, for example), to
 ensure that the directory data also reaches disk, in addition to the
-contents of the file.
+contents of the file. Certain filesystem options may also need to be
+set, such as C<data=journal> or C<data=ordered> on ext3, so that any
+crashes or unexpected glitches have less chance of unanticipated
+problems (such as the file write being ordered after the rename).
 
 =head1 SEE ALSO
 
@@ -750,13 +772,19 @@ Alternatives, depending on the need, include:
 
 L<IO::Atomic>, L<File::Transaction>, L<File::Transaction::Atomic>, L<Directory::Transactional>
 
+This isn't easy:
+
+L<http://danluu.com/file-consistency/>
+
 =head1 AUTHOR
 
 thrig - Jeremy Mates (cpan:JMATES) C<< <jmates at cpan.org> >>
 
+C<mtime> support contributed by Stijn De Weirdt.
+
 =head1 COPYRIGHT
 
-Copyright (C) 2009-2015 Jeremy Mates
+Copyright (C) 2009-2016 Jeremy Mates
 
 This module is free software; you can redistribute it and/or modify it
 under the Artistic License (2.0).
